import { Component, OnInit, Output, EventEmitter } from '@angular/core';
import { FormBuilder, FormGroup, Validators } from '@angular/forms';
import { MatDialogRef } from '@angular/material/dialog';
import { HttpClient, HttpHeaders } from '@angular/common/http';

@Component({
  selector: 'app-create-organizational-unit',
  templateUrl: './create-organizational-unit.component.html',
  styleUrls: ['./create-organizational-unit.component.css']
})
export class CreateOrganizationalUnitComponent implements OnInit {
  isLinear = true;
  generalFormGroup: FormGroup;
  additionalInfoFormGroup: FormGroup;
  networkSettingsFormGroup: FormGroup;
  classroomInfoFormGroup: FormGroup; 
  types: string[] = ['organizational-unit', 'classrooms-group', 'classroom', 'clients-group'];
  parentUnits: any[] = []; 

  @Output() unitAdded = new EventEmitter(); 

  constructor(
    private _formBuilder: FormBuilder,
    private dialogRef: MatDialogRef<CreateOrganizationalUnitComponent>,
    private http: HttpClient 
  ) {
    this.generalFormGroup = this._formBuilder.group({
      name: ['', Validators.required],
      parent: [''],
      description: [''],
      type: ['', Validators.required]
    });
    this.additionalInfoFormGroup = this._formBuilder.group({
      comments: [''],
    });
    this.networkSettingsFormGroup = this._formBuilder.group({
      proxy: [''],
      dns: [''],
      netmask: [''],
      router: [''],
      ntp: [''],
      p2pMode: [''],
      p2pTime: [0, Validators.min(0)],
      mcastIp: [''],
      mcastSpeed: [0, Validators.min(0)],
      mcastPort: [0, Validators.min(0)],
      mcastMode: [''],
      menu: ['', Validators.pattern('https?://.+')],
      hardwareProfile: ['', Validators.pattern('https?://.+')],
      validation: [false]
    });
    this.classroomInfoFormGroup = this._formBuilder.group({
      location: [''],
      projector: [false],
      board: [false],
      capacity: [0, Validators.min(0)],
      photo: ['', Validators.pattern('https?://.+')] 
    });
  }

  ngOnInit() {
    this.loadParentUnits(); 
  }

  loadParentUnits() {
    const url = 'http://127.0.0.1:8080/organizational-units?page=1&itemsPerPage=30';

    this.http.get<any>(url).subscribe(
      response => {
        this.parentUnits = response['hydra:member'];
      },
      error => {
        console.error('Error fetching parent units:', error);
      }
    );
  }

  onSubmit() {
    if (this.generalFormGroup.valid && this.additionalInfoFormGroup.valid && this.networkSettingsFormGroup.valid && (this.generalFormGroup.value.type !== 'classroom' || this.classroomInfoFormGroup.valid)) {
      const parentValue = this.generalFormGroup.value.parent;

      const formData: any = {
        name: this.generalFormGroup.value.name,
        parent: parentValue || null,
        description: this.generalFormGroup.value.description,
        comments: this.additionalInfoFormGroup.value.comments,
        type: this.generalFormGroup.value.type,
        networkSettings: {
          proxy: this.networkSettingsFormGroup.value.proxy,
          dns: this.networkSettingsFormGroup.value.dns,
          netmask: this.networkSettingsFormGroup.value.netmask,
          router: this.networkSettingsFormGroup.value.router,
          ntp: this.networkSettingsFormGroup.value.ntp,
          p2pMode: this.networkSettingsFormGroup.value.p2pMode,
          p2pTime: this.networkSettingsFormGroup.value.p2pTime,
          mcastIp: this.networkSettingsFormGroup.value.mcastIp,
          mcastSpeed: this.networkSettingsFormGroup.value.mcastSpeed,
          mcastPort: this.networkSettingsFormGroup.value.mcastPort,
          mcastMode: this.networkSettingsFormGroup.value.mcastMode,
          menu: this.networkSettingsFormGroup.value.menu,
          hardwareProfile: this.networkSettingsFormGroup.value.hardwareProfile,
          validation: this.networkSettingsFormGroup.value.validation
        }
      };

      if (this.generalFormGroup.value.type === 'classroom') {
        formData.location = this.classroomInfoFormGroup.value.location;
        formData.projector = this.classroomInfoFormGroup.value.projector;
        formData.board = this.classroomInfoFormGroup.value.board;
        formData.capacity = this.classroomInfoFormGroup.value.capacity;
        formData.photo = this.classroomInfoFormGroup.value.photo; 
      }

      console.log('POST data:', formData);
      const postUrl = 'http://127.0.0.1:8080/organizational-units';
      const headers = new HttpHeaders({ 'Content-Type': 'application/json' });

      this.http.post<any>(postUrl, formData, { headers }).subscribe(
        response => {
          console.log('POST successful:', response);
          this.unitAdded.emit();
          this.dialogRef.close(); 
        },
        error => {
          console.error('Error al realizar POST:', error);
        }
      );
    }
  }

  onNoClick(): void {
    this.dialogRef.close();
  }
}
